import React from 'react';
import { ArrowDown, ArrowLeft, ArrowRight, ArrowUp, EllipsisVertical, TrashBin, Xmark, } from '@gravity-ui/icons';
import { Button, DropdownMenu, Icon } from '@gravity-ui/uikit';
import { findChildren, findParentNodeClosestToPos } from 'prosemirror-utils';
import { createPortal } from 'react-dom';
import { cn } from '../../../../../classname';
import { bindActions } from '../../../../../core';
import { i18n } from '../../../../../i18n/yfm-table';
import { ErrorLoggerBoundary } from '../../../../../react-utils/ErrorBoundary';
import { getTableDimensions, isTableCellNode, isTableNode, isTableRowNode, } from '../../../../../table-utils';
import { getReactRendererFromState } from '../../../../behavior/ReactRenderer';
import { controlActions } from './actions';
import './yfmTableCellView.css';
export const yfmTableCellCn = cn('table-cell-view');
const b = yfmTableCellCn;
const Controls = function Controls({ columnNumber, rowNumber, view, getParentTable, isInFirstRow, isFirstInRow, }) {
    var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k;
    const actions = bindActions(controlActions)(view);
    const controls = [];
    if (isFirstInRow) {
        controls.push(React.createElement(DropdownMenu, { key: 1, switcherWrapperClassName: b('left-button'), switcher: React.createElement(Button, { view: 'outlined', size: 's' },
                React.createElement(Icon, { data: EllipsisVertical })), items: [
                {
                    text: i18n('row.add.before'),
                    action: () => {
                        var _a;
                        return actions.appendRow.run({
                            tablePos: (_a = getParentTable()) === null || _a === void 0 ? void 0 : _a.pos,
                            rowNumber,
                            direction: 'before',
                        });
                    },
                    icon: React.createElement(Icon, { data: ArrowUp }),
                    disabled: !actions.appendRow.isEnable({
                        tablePos: (_a = getParentTable()) === null || _a === void 0 ? void 0 : _a.pos,
                        rowNumber,
                        direction: 'before',
                    }),
                },
                {
                    text: i18n('row.add.after'),
                    action: () => {
                        var _a;
                        return actions.appendRow.run({
                            tablePos: (_a = getParentTable()) === null || _a === void 0 ? void 0 : _a.pos,
                            rowNumber,
                            direction: 'after',
                        });
                    },
                    icon: React.createElement(Icon, { data: ArrowDown }),
                    disabled: !actions.appendRow.isEnable({
                        tablePos: (_b = getParentTable()) === null || _b === void 0 ? void 0 : _b.pos,
                        rowNumber,
                        direction: 'after',
                    }),
                },
                {
                    text: i18n('row.remove'),
                    action: () => {
                        var _a;
                        return actions.deleteRow.run({
                            rowNumber,
                            tablePos: (_a = getParentTable()) === null || _a === void 0 ? void 0 : _a.pos,
                        });
                    },
                    icon: React.createElement(Icon, { data: Xmark }),
                    disabled: !actions.deleteRow.isEnable({
                        rowNumber,
                        tablePos: (_c = getParentTable()) === null || _c === void 0 ? void 0 : _c.pos,
                    }),
                },
                {
                    theme: 'danger',
                    text: i18n('table.remove'),
                    action: () => {
                        var _a, _b;
                        return actions.deleteTable.run({
                            tablePos: (_a = getParentTable()) === null || _a === void 0 ? void 0 : _a.pos,
                            tableNode: (_b = getParentTable()) === null || _b === void 0 ? void 0 : _b.node,
                        });
                    },
                    icon: React.createElement(Icon, { data: TrashBin }),
                    disabled: !actions.deleteTable.isEnable({
                        tablePos: (_d = getParentTable()) === null || _d === void 0 ? void 0 : _d.pos,
                        tableNode: (_e = getParentTable()) === null || _e === void 0 ? void 0 : _e.node,
                    }),
                },
            ] }));
    }
    if (isInFirstRow) {
        controls.push(React.createElement(DropdownMenu, { key: 2, switcherWrapperClassName: b('upper-button'), switcher: React.createElement(Button, { view: 'outlined', size: 's' },
                React.createElement(Icon, { data: EllipsisVertical })), items: [
                {
                    text: i18n('column.add.before'),
                    action: () => {
                        var _a;
                        return actions.appendColumn.run({
                            tablePos: (_a = getParentTable()) === null || _a === void 0 ? void 0 : _a.pos,
                            columnNumber,
                            direction: 'before',
                        });
                    },
                    icon: React.createElement(Icon, { data: ArrowLeft }),
                    disabled: !actions.appendColumn.isEnable({
                        tablePos: (_f = getParentTable()) === null || _f === void 0 ? void 0 : _f.pos,
                        columnNumber,
                        direction: 'before',
                    }),
                },
                {
                    text: i18n('column.add.after'),
                    action: () => {
                        var _a;
                        return actions.appendColumn.run({
                            tablePos: (_a = getParentTable()) === null || _a === void 0 ? void 0 : _a.pos,
                            columnNumber,
                            direction: 'after',
                        });
                    },
                    icon: React.createElement(Icon, { data: ArrowRight }),
                    disabled: !actions.appendColumn.isEnable({
                        tablePos: (_g = getParentTable()) === null || _g === void 0 ? void 0 : _g.pos,
                        columnNumber,
                        direction: 'after',
                    }),
                },
                {
                    text: i18n('column.remove'),
                    action: () => {
                        var _a;
                        return actions.deleteColumn.run({
                            columnNumber,
                            tablePos: (_a = getParentTable()) === null || _a === void 0 ? void 0 : _a.pos,
                        });
                    },
                    icon: React.createElement(Icon, { data: Xmark }),
                    disabled: !actions.deleteColumn.isEnable({
                        columnNumber,
                        tablePos: (_h = getParentTable()) === null || _h === void 0 ? void 0 : _h.pos,
                    }),
                },
                {
                    theme: 'danger',
                    text: i18n('table.remove'),
                    action: () => {
                        var _a, _b;
                        return actions.deleteTable.run({
                            tablePos: (_a = getParentTable()) === null || _a === void 0 ? void 0 : _a.pos,
                            tableNode: (_b = getParentTable()) === null || _b === void 0 ? void 0 : _b.node,
                        });
                    },
                    icon: React.createElement(Icon, { data: TrashBin }),
                    disabled: !actions.deleteTable.isEnable({
                        tablePos: (_j = getParentTable()) === null || _j === void 0 ? void 0 : _j.pos,
                        tableNode: (_k = getParentTable()) === null || _k === void 0 ? void 0 : _k.node,
                    }),
                },
            ] }));
    }
    return controls.length ? React.createElement(React.Fragment, null, controls) : null;
};
export const yfmTableCellView = (node, view, getPos) => {
    const dom = document.createElement('td');
    const contentDOM = document.createElement('div');
    const control = document.createElement('span');
    control.setAttribute('style', 'width: 0; height: 0; float: left;');
    dom.setAttribute('style', 'position: relative; overflow: visible;');
    const getParentTable = () => findParentNodeClosestToPos(view.state.doc.resolve(getPos()), isTableNode);
    const parentTable = getParentTable();
    // @ts-expect-error
    if (!parentTable)
        return {};
    const rowNodes = findChildren(parentTable === null || parentTable === void 0 ? void 0 : parentTable.node, isTableRowNode);
    const relativeCelPos = getPos() - (parentTable === null || parentTable === void 0 ? void 0 : parentTable.pos);
    const rowNumber = rowNodes.findIndex((_v, i) => {
        var _a;
        return relativeCelPos > rowNodes[i].pos &&
            relativeCelPos < (((_a = rowNodes[i + 1]) === null || _a === void 0 ? void 0 : _a.pos) || Number.MAX_SAFE_INTEGER);
    });
    const parentRow = rowNodes[rowNumber];
    const columnNumber = findChildren(parentRow.node, isTableCellNode).findIndex((c) => {
        return c.pos === relativeCelPos - parentRow.pos - 2;
    });
    const isFirstInRow = rowNodes.findIndex((r) => r.pos === relativeCelPos - 2) >= 0;
    dom.appendChild(contentDOM);
    dom.appendChild(control);
    const renderItem = getReactRendererFromState(view.state).createItem('yfm-table-cell-view', () => createPortal(React.createElement(ErrorLoggerBoundary, null,
        React.createElement(Controls, { columnNumber: columnNumber, rowNumber: rowNumber, isFirstInRow: isFirstInRow, isInFirstRow: rowNumber === 0, getParentTable: getParentTable, view: view })), control));
    contentDOM.setAttribute('data-row-number', String(rowNumber));
    contentDOM.setAttribute('data-col-number', String(columnNumber));
    return {
        dom,
        contentDOM,
        destroy() {
            renderItem.remove();
        },
        ignoreMutation(mutation) {
            return mutation.type === 'childList' && mutation.target === control;
        },
        update(n) {
            const { rows: nRows, cols: nCols } = getTableDimensions(n);
            const { rows, cols } = getTableDimensions(node);
            return !(rows !== nRows || cols !== nCols);
        },
    };
};
